<?php

namespace Drupal\cmlmigrations\Plugin\migrate\source;

use Drupal\cmlmigrations\Utility\MigrationsSourceBase;
use Drupal\paragraphs\Entity\Paragraph;

/**
 * Source for CSV.
 *
 * @MigrateSource(
 *   id = "cml_commerce_product_variation_rest"
 * )
 */
class CommerceProductVariationRest extends MigrationsSourceBase {

  /**
   * {@inheritdoc}
   */
  public function getRows() {
    $rows = [];
    $stores = $this->mapStores();
    $source = \Drupal::service('cmlapi.parser_rests')->parse();
    if ($source) {
      $k = 0;
      foreach ($source as $key => $row) {
        if ($k++ < 50 || !$this->uipage) {
          $id = $row['Id'];
          $rows[$id] = [
            'uuid' => $id,
          ];
          $variationId = $this->updateMapping($id);
          if ($variationId) {
            foreach ($row['Ostatki'][0]['Остаток'] as $value) {
              if (!empty($stores[$value['Склад']['Ид']])) {
                $paragraph = $this->hasStore($stores[$value['Склад']['Ид']], $value['Склад']['Количество'], $variationId);
                $rows[$id]['field_stores'][] = $paragraph;
              }
            }
          }
        }
      }
    }
    $this->debug = TRUE;
    return $rows;
  }

  /**
   * Update mapping.
   */
  private function updateMapping($sourceId) {
    $destid = FALSE;
    $querySource = \Drupal::database()->select('migrate_map_cml_product_variation', 'maps_source');
    $querySource->fields('maps_source', [
      'source_ids_hash',
      'sourceid1',
      'destid1',
    ])
      ->condition('sourceid1', $sourceId);
    $resSource = $querySource->execute();

    if ($resSource && \Drupal::database()->schema()->tableExists('migrate_map_cml_product_variation_rest')) {
      foreach ($resSource as $rowSource) {
        $destid = $rowSource->destid1;
        $upsert = \Drupal::database()->upsert('migrate_map_cml_product_variation_rest')
          ->fields([
            'source_ids_hash' => $rowSource->source_ids_hash,
            'sourceid1' => $rowSource->sourceid1,
            'destid1' => $destid,
          ])
          ->key('sourceid1')
          ->execute();
      }
    }
    return $destid;
  }

  /**
   * HasStore.
   */
  private function hasStore($store_id, $quantity, $variationId) {
    $result = FALSE;
    $storage = \Drupal::entityTypeManager()->getStorage('paragraph');
    $ids = $storage
      ->getQuery()
      ->condition('status', 1)
      ->condition('type', 'stores')
      ->condition('parent_type', 'commerce_product_variation')
      ->condition('parent_field_name', 'field_stores')
      ->condition('parent_id', $variationId)
      ->condition('field_store_name', $store_id)
      ->execute();
    if ($ids) {
      foreach ($storage->loadMultiple($ids) as $paragraph) {
        $paragraph->set('field_stock', $quantity);
        $paragraph->save();
      }
    }
    else {
      $paragraph = Paragraph::create([
        'type' => 'stores',
        'parent_id' => $variationId,
        'parent_type' => 'commerce_product_variation',
        'parent_field_name' => 'field_stores',
        'field_stock' => $quantity,
        'field_store_name' => [
          'target_id' => $store_id,
        ],
      ]);
      $paragraph->save();
    }
    $result = [
      'target_id' => $paragraph->id(),
      'target_revision_id' => $paragraph->getRevisionId(),
    ];
    return $result;
  }

  /**
   * Map Stores.
   */
  private function mapStores() {
    $storage = \Drupal::entityTypeManager()->getStorage('taxonomy_term');
    $ids = $storage
      ->getQuery()
      ->condition('vid', 'stores')
      ->condition('status', 1)
      ->execute();
    $stores = [];
    if ($ids) {
      foreach ($storage->loadMultiple($ids) as $store) {
        $stores[$store->field_store_1s->value] = $store->id();
      }
    }
    return $stores;
  }

}
